@@ -27,7 +27,8 @@
 
 import wx
 import wx.stc
-from Numeric import *
+#from Numeric import *
+from threading import *
 from scipy import integrate
 from string import *
 from RandomArray import *
@@ -36,10 +37,13 @@ import matplotlib
 matplotlib.use('WXAgg')
 from pylab import *
 from matplotlib.cbook import *
-import os
+from xml.dom import Node
+
+import os,sys
 import wxFrame2, about
 
 import uncertaintyMiniFrame
+from odexml import Model
 import PlotFigure as PF
 from Bayes import Melding as meld
 from time import *
@@ -126,7 +130,7 @@ class wxFrame1(wx.Frame):
     def _init_coll_toolBar1_Tools(self, parent):
         # generated method, don't edit
 
-        parent.AddTool(bitmap=wx.Bitmap('timemanagement_section.png',
+        parent.AddTool(bitmap=wx.Bitmap('/home/fccoelho/Documents/Projects_software/modelbuilder/model-builder/timemanagement_section.png',
               wx.BITMAP_TYPE_PNG), id=wxID_WXFRAME1TOOLBAR1TOOLS0,
               isToggle=False, longHelpString='Start your simulation.',
               pushedBitmap=wx.NullBitmap, shortHelpString='Start')
@@ -175,7 +179,7 @@ class wxFrame1(wx.Frame):
     def _init_ctrls(self, prnt):
         # generated method, don't edit
         wx.Frame.__init__(self, id=wxID_WXFRAME1, name='', parent=prnt,
-              pos=wx.Point(336, 171), size=wx.Size(680, 457),
+              pos=wx.Point(337, 157), size=wx.Size(680, 457),
               style=wx.DEFAULT_FRAME_STYLE, title='Model Builder - ODE')
         self._init_utils()
         self.SetClientSize(wx.Size(680, 457))
@@ -191,9 +195,10 @@ class wxFrame1(wx.Frame):
         self.SetStatusBar(self.statusBar1)
 
         self.toolBar1 = wx.ToolBar(id=wxID_WXFRAME1TOOLBAR1, name='toolBar1',
-              parent=self, pos=wx.Point(0, 0), size=wx.Size(144, 42),
-              style=wx.TB_HORIZONTAL | wx.NO_BORDER)
+              parent=self, pos=wx.Point(0, 0), size=wx.Size(142, 50),
+              style=wx.TRANSPARENT_WINDOW | wx.TB_HORIZONTAL | wx.NO_BORDER)
         self.toolBar1.SetToolTipString('')
+        self.toolBar1.SetThemeEnabled(True)
 
         self.staticText1 = wx.StaticText(id=wxID_WXFRAME1STATICTEXT1,
               label='Differential Equations:', name='staticText1', parent=self,
@@ -329,38 +334,54 @@ class wxFrame1(wx.Frame):
         self.FileName = None
         self.ModLoaded = None #no model has been loaded
         self.Neq = None
-        self.modict = {'name':None, 'type':'ODE'}
+        self.modict = {'modelname':''}
+        self.modtree = Model() #Object containing dom tree with model specification
         self.modbuilt = 0 #model has never been built
         self.modRan = 0 #model has never been ran
         self.plot = 0 #no plot has ever been generated
         self.gauge1.SetRange(100)
         self.uncertainty = 0 # uncertainty analysis is off
+        
 
     def OnMenu1items0Menu(self, event):
+        """
+        Load a model file.
+        """
         os.chdir(os.getcwd())
         dlg = wx.FileDialog(self, "Choose a file", ".", "", "*.ode", wx.OPEN)
         try:
             if dlg.ShowModal() == wx.ID_OK:
                 filename = dlg.GetPath()
-                f = open(filename)
-                self.modict = pickle.load(f)
-                self.eqnEdit.SetValue(self.modict['equations'])
-                self.textCtrl2.SetValue(str(self.modict['start']))
-                self.textCtrl3.SetValue(str(self.modict['end']))
-                self.textCtrl4.SetValue(str(self.modict['step']))
-                self.textCtrl1.SetValue(str(self.modict['init']))
-                self.parEdit.SetValue(self.modict['parameters'])
-                self.FileName = filename
-                self.SetTitle('Model Builder - '+filename)
-                self.gauge1.SetValue(0)
-                f.close()
+                a = self.modtree.readFile(filename)
+                if a: #model is not a xml file
+                    f = open(filename)
+                    self.modict = pickle.load(f)
+                    self.fillGui()
+                    self.FileName = filename
+                    self.SetTitle('Model Builder - '+filename)
+                    f.close()
+                else:
+                    self.modict.update(self.modtree.modict)
+                    self.fillGui()
                 self.ModLoaded = 1
         finally:
             dlg.Destroy()
 
-
-
+    def fillGui(self):
+        """ 
+        fill GUI from model dictionary
+        """
+        self.eqnEdit.SetValue(self.modict['equations'])
+        self.textCtrl2.SetValue(str(self.modict['start']))
+        self.textCtrl3.SetValue(str(self.modict['end']))
+        self.textCtrl4.SetValue(str(self.modict['step']))
+        self.textCtrl1.SetValue(str(self.modict['init']))
+        self.parEdit.SetValue(self.modict['parameters'])
+    
     def OnMenu1items1Menu(self, event):
+        """
+        Save model.
+        """
         if self.FileName == None:
             return self.OnMenu1items2Menu(event)
         else:
@@ -369,14 +390,22 @@ class wxFrame1(wx.Frame):
             self.BuildModel(r=0) # call BuildModel without running the model
             pickle.dump(self.modict, f)
             f.close()
+            # Saving in xml format
+            self.modtree.Create(**self.modict)
+            self.modetree.saveFile(filename)
 
 
     def OnMenu1items2Menu(self, event):
-        
+        """
+        Save as menu entry
+        """
         dlg = wx.FileDialog(self, "Save File As", ".", "", "*.ode", wx.SAVE)
         try:
             if dlg.ShowModal() == wx.ID_OK:
                 filename = dlg.GetPath()
+                self.modict['modelname'] = filename.split('/')[-1]
+                if not filename.endswith('.ode'):
+                    filename += '.ode' 
                 f=open(filename, 'w')
                 if self.modbuilt: #check to see if model's dictionary has been built
                     pickle.dump(self.modict, f)
@@ -386,14 +415,22 @@ class wxFrame1(wx.Frame):
                 f.close()
                 self.FileName = filename
                 self.SetTitle('Model Builder - '+filename)
+                # Saving in xml format
+                print self.modict.keys()
+                self.modtree.Create(**self.modict)
+                self.modtree.saveFile(filename)
 
         finally:
             dlg.Destroy()
 
 
     def OnMenu1items3Menu(self, event):
+        """
+        Close model menu entry.
+        """
         self.FileName = None
         self.ModLoaded = None
+        self.modict = {}
         self.eqnEdit.SetValue('')
         self.textCtrl2.SetValue('')
         self.textCtrl3.SetValue('')
@@ -536,9 +573,7 @@ class wxFrame1(wx.Frame):
 
 
         self.modbuilt = 0
-        self.sw = wx.StopWatch()
-        self.gauge1.SetValue(0)
-        self.sw.Start()
+        self.updateGauge(0)
         if self.uncertainty == 0:
             self.BuildModel(r=1) #regular run
             self.modRan = 1 #set the flag indicating model has been run
@@ -555,22 +590,19 @@ class wxFrame1(wx.Frame):
                 finally:
                     dlg.Destroy()
 
-        if self.modRan:
-            t = gmtime(self.sw.Time()/1000)
-
-            dlg = wx.MessageDialog(self, 'The simulation was completed in '+str(t[3])+' hours, '+str(t[4])+' minutes and '+str(t[5])+' seconds.',
-          'Time Elapsed', wx.OK | wx.ICON_INFORMATION)
-            try:
-                dlg.ShowModal()
-            finally:
-                dlg.Destroy()
-            if self.uncertainty == 1:
-                self.uncertaintyPanel.statusPanel.AppendText('The simulation was completed in:\n '+str(t[3])+' hours, '+str(t[4])+' minutes and '+str(t[5])+' seconds.')
-
 
 
     def OnToolbar1tools0ToolRclicked(self, event):
         event.Skip()
+        
+    def updateGauge(self,t):
+        """
+        This function simply updates the progress gauge
+        """
+        pd = (100*t)/(float(strip(str(self.textCtrl3.GetValue())))-float(strip(str(self.textCtrl2.GetValue())))) #calcutates percentage done
+        self.gauge1.SetValue(int(pd))
+        #self.gauge1.SetToolTipString(str(pd)+'%')
+        self.gauge1.Refresh() #update the gauge
 
     def Equations(self, y, t):
         """
@@ -579,10 +611,7 @@ class wxFrame1(wx.Frame):
 
         returns ydot
         """
-##        pd = (100*t)/(float(strip(str(self.textCtrl3.GetValue())))-float(strip(str(self.textCtrl2.GetValue())))) #calcutates percentage done
-##        self.gauge1.SetValue(int(pd))
-##        self.gauge1.SetToolTipString(str(pd)+'%')
-##        self.gauge1.Refresh() #update the gauge
+        self.updateGauge(t)
 
         Neq=self.Neq
         Npar = self.Npar
@@ -594,7 +623,7 @@ class wxFrame1(wx.Frame):
     #---Create Parameter Array----------------------------------------------------------------------------
         if self.uncertainty == 0:
             pars = strip(par.GetValue()).split('\n')
-            if len(pars) != 0:
+            if pars[0]: #only if there is at least one parameter
                 for j in xrange(len(pars)):
                     try:
                         p[j] = eval(pars[j]) #initialize parameter values
@@ -767,6 +796,7 @@ class wxFrame1(wx.Frame):
         """
         Constructs the model from the input fields and runs it if r==1
         """
+        self.modict = {}
         self.modict['equations'] = self.eqnEdit.GetValue() #put equations into model's dictionary
         self.modict['parameters'] = self.parEdit.GetValue() # put parameters into model's dictionary
         t_start = float(strip(str(self.textCtrl2.GetValue())))
@@ -797,6 +827,9 @@ class wxFrame1(wx.Frame):
 
         self.Neq = Neq#to be used by Equations
         if r==1:
+            self.sw = wx.StopWatch()
+            self.gauge1.SetValue(0)
+            self.sw.Start()
             #-----------------------------------------------------
             if self.uncertainty == 0:#regular (single) run
                 if not self.parEdit.GetValue() == '':#get Number of  Parameters
@@ -835,6 +868,17 @@ class wxFrame1(wx.Frame):
                 (x,runs_byvar) = self.plotMelding(t_range,t_courseList,tit='Median Runs and 95% intervals')
                 self.Bmeld(x,runs_byvar) # Perform the rest of the melding calculations
                 self.modict['meldruns'] = t_courseList
+                
+            t = gmtime(self.sw.Time()/1000)
+
+            dlg = wx.MessageDialog(self, 'The simulation was completed in '+str(t[3])+' hours, '+str(t[4])+' minutes and '+str(t[5])+' seconds.',
+          'Time Elapsed', wx.OK | wx.ICON_INFORMATION)
+            try:
+                dlg.ShowModal()
+            finally:
+                dlg.Destroy()
+            if self.uncertainty == 1:
+                self.uncertaintyPanel.statusPanel.AppendText('The simulation was completed in:\n '+str(t[3])+' hours, '+str(t[4])+' minutes and '+str(t[5])+' seconds.')
 #-------------------------------------------------------------------------------
 
         self.modbuilt = 1
@@ -1025,3 +1069,71 @@ class wxFrame1(wx.Frame):
             self.uncertaintyPanel.Close()
         except (AttributeError, NameError):
             pass
+        sys.exit()
+class Future:
+    """
+    By David Perry - http://aspn.activestate.com/ASPN/Cookbook/Python/Recipe/84317
+    To run a function in a separate thread, simply put it in a Future:
+
+    >>> A=Future(longRunningFunction, arg1, arg2 ...)
+
+    It will continue on its merry way until you need the result of your function. 
+    You can read the result by calling the Future like a function, for example:
+
+    >>> print A()
+
+    If the Future has completed executing, the call returns immediately. 
+    If it is still running, then the call blocks until the function completes. 
+    The result of the function is stored in the Future, so subsequent calls to 
+    it return immediately.
+
+    A few caveats:
+    Since one wouldn't expect to be able to change the result of a function, 
+    Futures are not meant to be mutable. This is enforced by requiring the 
+    Future to be "called", rather than directly reading __result. If desired, 
+    stronger enforcement of this rule can be achieved by playing 
+    with __getattr__ and __setattr__.
+
+    The Future only runs the function once, no matter how many times you 
+    read it. You will have to re-create the Future if you want to re-run your 
+    function; for example, if the function is sensitive to the time of day.
+
+    For more information on Futures, and other useful parallel programming 
+    constructs, read Gregory V. Wilson's _Practical Parallel Programming_.
+    """
+    def __init__(self,func,*param):
+        # Constructor
+        self.__done=0
+        self.__result=None
+        self.__status='working'
+
+        self.__C=Condition()   # Notify on this Condition when result is ready
+
+        # Run the actual function in a separate thread
+        self.__T=Thread(target=self.Wrapper,args=(func,param))
+        self.__T.setName("FutureThread")
+        self.__T.start()
+
+    def __repr__(self):
+        return '<Future at '+hex(id(self))+':'+self.__status+'>'
+
+    def __call__(self):
+        self.__C.acquire()
+        while self.__done==0:
+            self.__C.wait()
+        self.__C.release()
+        # We deepcopy __result to prevent accidental tampering with it.
+        a=copy.deepcopy(self.__result)
+        return a
+
+    def Wrapper(self, func, param):
+        # Run the actual function, and let us housekeep around it
+        self.__C.acquire()
+        try:
+            self.__result=func(*param)
+        except:
+            self.__result="Exception raised within Future"
+        self.__done=1
+        self.__status=`self.__result`
+        self.__C.notify()
+        self.__C.release()
